UNIT LRLLevels;

INTERFACE

USES DOSFileAccess,LRLRoutines,LRLSprites;

TYPE
	TLRLPlayerPosition = RECORD
		x,y: BYTE;
		xoffs,yoffs: SHORTINT;
	END;

	TLRLPlayer = RECORD
		Command: BYTE;
		{ pictures:
			1 - running left <-
			2 - running right ->
			3 - climbing up ^
			4 - climbing down v
			5 - falling
			6 - ~~~~~ left <-
			7 - ~~~~~ right ->
			8 - firing left <-
			9 - firing right ->
		}
		NewCommandWas: BOOLEAN;
		NewCommand: BYTE;
		Position: TLRLPlayerPosition;
		Sprite: BYTE;
		SpriteData: BYTE;
		Controller: BYTE;
		{
			controllers:
			0 - not playing
			1 - human/keyboard
			2 - computer
		}
		Prizes: BYTE;
		{
			max 1 if computer player
			a) computer player leaves prize if falling into hole
			b) takes prize if he has no prizes
		}
		Colour: BYTE;
	END;

	TLRLBrick = RECORD
		Image: BYTE;
		Count: BYTE;
		Flags: BYTE;
		{ flags:
			bit 0 - needed to animate this brick 5 sprites then pause
				and then finnally 5 sprites
			bit 1 - set if fatal brick
			bit 2 - set if allowable to jump
			bit 3 - allowable to walk thru
			bit 4 - hidden
			bit 5 - background
			bit 6 - wait now
			bit 7 - not draw it
		}
		IdleCount: BYTE;
	END;

	TLRLLevel = RECORD
		Field: ARRAY[1..30,1..16] OF TLRLBrick;
		Player: ARRAY[1..20] OF TLRLPlayer;
	END;

CONST
	BrickFlags: ARRAY[1..20] OF BYTE =
	(48,4+8+32+128,
	49,8+32,
	50,4+8+32,
	51,4+8+32,
	52,2,
	53,4,
	54,4+8,
	55,2,
	56,2,
	65,4+8+16+32);

CONST
	KeyboardControls: ARRAY[1..24] OF WORD =
	($4800,1,3,
	$5000,1,4,
	$4B00,1,1,
	$4D00,1,2,
	$332C,1,6,
	$342E,1,7,
	$333C,1,6,
	$343E,1,7);
	ControlNumber = 8;

VAR
	ShowLives: BOOLEAN;
	ShowScore: BOOLEAN;
	ShowLevel: BOOLEAN;
	LRLLevel: TLRLLevel;
	LRLLives: INTEGER;
	LRLScore: LONGINT;
	LRLCLevel: WORD;
	ComputerTurn: WORD;
	ComputerReaction: WORD;
	TimeToRefresh: BOOLEAN;
	RefreshDelay: WORD;
	RefreshRemain: WORD;
	OldTimer: POINTER;
	TotalPrizes: INTEGER;
	GameStarted: BOOLEAN;
	EndOfGame: BOOLEAN;
	GameResult: WORD;
	Paused: BOOLEAN;

PROCEDURE LRLLoadLevel(Number: BYTE);
PROCEDURE LRLUpdatePlayers;
PROCEDURE LRLDrawOrnamental(x1,y1,x2,y2,ornament: BYTE);
FUNCTION LRLPlayLevel(Number: BYTE): WORD;
FUNCTION LRLLevelCount: WORD;
PROCEDURE LRLDeleteLevel(Count: WORD);
PROCEDURE LRLInsertLevel(After: WORD);
PROCEDURE LRLSaveLevel(Count: WORD);

IMPLEMENTATION

CONST
	LevelFileName = 'LRL.LEV';
	LevelFileHeader: STRING[24] = 'Lode Runner Live Levels'#26;

FUNCTION LRLLevelCount: WORD;
VAR
	LevelFile: TDFAFileHandle;
	c,k: WORD;
BEGIN
	c:=0;
	DFAFileOpen(LevelFile,LevelFileName,DFAAccessRead);
	DFAFilePositionSet(LevelFile,24,DFASeekFromStart);
	DFAFileRead(LevelFile,c,1,k);
	LRLLevelCount:=c;
	DFAFileClose(LevelFile);
END;

PROCEDURE LRLSaveLevel(Count: WORD);
VAR
	LevelFile: TDFAFileHandle;
	i,j: INTEGER;
	k: WORD;
	b: POINTER;
BEGIN
	GETMEM(b,480);
	IF (Count=0) OR (Count>LRLLevelCount) THEN EXIT;
	DFAFileOpen(LevelFile,LevelFileName,DFAAccessReadWrite);
	DFAFilePositionSet(LevelFile,LONGINT(25+520*(LONGINT(Count)-1)),DFASeekFromStart);
	FOR i:=1 TO 10 DO BEGIN
		DataBytePut(b^,(i-1)*4,LRLLevel.Player[i].Position.x);
		DataBytePut(b^,(i-1)*4+1,LRLLevel.Player[i].Position.y);
		DataBytePut(b^,(i-1)*4+2,LRLLevel.Player[i].Colour);
		DataBytePut(b^,(i-1)*4+3,LRLLevel.Player[i].Controller);
	END;
	DFAFileWrite(LevelFile,b^,40,k);
	FOR i:=1 TO 16 DO
		FOR j:=1 TO 30 DO
			DataBytePut(b^,(i-1)*30+j-1,LRLLevel.Field[j,i].Image+47);
	DFAFileWrite(LevelFile,b^,480,k);
	DFAFileClose(LevelFile);
	FREEMEM(b,480);
END;

PROCEDURE LRLDeleteLevel(Count: WORD);
VAR
	Buffer: POINTER;
	LevelFile: TDFAFileHandle;
	i,j: INTEGER;
	l: LONGINT;
	k: WORD;
BEGIN
	GETMEM(Buffer,1000);
	j:=LRLLevelCount;
	IF (j<Count) OR (j<2) OR (Count=0) THEN EXIT;
	DFAFileOpen(LevelFile,LevelFileName,DFAAccessReadWrite);
	FOR l:=Count+1 TO j DO BEGIN
		DFAFilePositionSet(LevelFile,LONGINT(25+520*(LONGINT(l)-1)),DFASeekFromStart);
		DFAFileRead(LevelFile,Buffer^,520,k);
		DFAFilePositionSet(LevelFile,LONGINT(25+520*(LONGINT(l-1)-1)),DFASeekFromStart);
		DFAFileWrite(LevelFile,Buffer^,520,k);
	END;
	DFAFilePositionSet(LevelFile,24,DFASeekFromStart);
	DEC(j);
	DFAFileWrite(LevelFile,j,1,k);
	DFAFilePositionSet(LevelFile,DFAFileSize(LevelFile)-520,DFASeekFromStart);
	DFAFileTruncate(LevelFile);
	DFAFileClose(LevelFile);
	FREEMEM(Buffer,1000);
END;

PROCEDURE LRLInsertLevel(After: WORD);
VAR
	Buffer: POINTER;
	LevelFile: TDFAFileHandle;
	i,j: INTEGER;
	l: LONGINT;
	k: WORD;
BEGIN
	GETMEM(Buffer,1000);
	j:=LRLLevelCount;
	IF (After>j) OR (After=0) THEN EXIT;
	DFAFileOpen(LevelFile,LevelFileName,DFAAccessReadWrite);
	FOR l:=j DOWNTO After+1 DO BEGIN
		DFAFilePositionSet(LevelFile,LONGINT(25+520*(LONGINT(l)-1)),DFASeekFromStart);
		DFAFileRead(LevelFile,Buffer^,520,k);
		DFAFilePositionSet(LevelFile,LONGINT(25+520*(LONGINT(l+1)-1)),DFASeekFromStart);
		DFAFileWrite(LevelFile,Buffer^,520,k);
	END;
	DFAFilePositionSet(LevelFile,24,DFASeekFromStart);
	INC(j);
	DFAFileWrite(LevelFile,j,1,k);
	DFAFilePositionSet(LevelFile,LONGINT(25+520*(LONGINT(After+1)-1)),DFASeekFromStart);
	DataFill(Buffer^,40,0,0);
	DataFill(Buffer^,480,48,40);
	DFAFileWrite(LevelFile,Buffer^,520,k);
	DFAFileClose(LevelFile);
	FREEMEM(Buffer,1000);
END;

PROCEDURE LRLLoadLevel(Number: BYTE);
VAR
	LevelFile: TDFAFileHandle;
	InBuffer: POINTER;
	i,j,k,l,x,y: WORD;
	a,b,c: BYTE;
BEGIN
	TotalPrizes:=0;
	GETMEM(InBuffer,$FFF0);
	DFAFileOpen(LevelFile,LevelFileName,DFAAccessRead);
	IF DFALastResult(LevelFile)<>0 THEN BEGIN
		WRITELN('Error: Cannot open file with levels!');
		WRITELN('訡:   䠩 ஢!');
		FHALT;
	END;
	DFAFileRead(LevelFile,InBuffer^,24,k);
	DFAFileRead(LevelFile,c,1,k);
	IF (c=0) OR (DFALastResult(LevelFile)<>0) OR (NOT DataIdentical(InBuffer^,LevelFileHeader[1],24,0,0)) THEN BEGIN
		WRITELN('Error: Invalid or damaged file with levels!');
		WRITELN('訡:   ० 䠩 ஢!');
		FHALT;
	END;
	IF (Number=0) OR (Number>c) THEN Number:=1;
	DFAFilePositionSet(LevelFile,LONGINT(25+520*(LONGINT(Number)-1)),DFASeekFromStart);
	DFAFileRead(LevelFile,InBuffer^,40,k);
	FOR i:=1 TO 10 DO
		WITH LRLLevel.Player[i] DO BEGIN
			Command:=10;
			NewCommandWas:=FALSE;
			NewCommand:=10;
			Position.x:=DataByteGet(InBuffer^,(i-1)*4+0);
			Position.y:=DataByteGet(InBuffer^,(i-1)*4+1);
			Position.xoffs:=0;
			Position.yoffs:=0;
			Sprite:=1;
			SpriteData:=1;
			Controller:=DataByteGet(InBuffer^,(i-1)*4+3);
			Prizes:=0;
			Colour:=DataByteGet(InBuffer^,(i-1)*4+2);
	END;
	DFAFileRead(LevelFile,InBuffer^,480,k);
	FOR i:=1 TO 16 DO
		FOR j:=1 TO 30 DO
			WITH LRLLevel.Field[j,i] DO BEGIN
				a:=DataByteGet(InBuffer^,(i-1)*30+(j-1));
				FOR b:=1 TO 10 DO
					IF BrickFlags[b*2-1]=a THEN Flags:=BrickFlags[b*2];
				Count:=1;
				IF a<64 THEN
					a:=a-47
				ELSE
					a:=a-63;
				Image:=a;
				IdleCount:=0;
				IF Image=4 THEN INC(TotalPrizes);
			END;
	DFAFileRead(LevelFile,InBuffer^,480,k);
	DFAFileClose(LevelFile);
	LRLCLevel:=Number;
	FREEMEM(InBuffer,$FFF0);
END;

PROCEDURE LRLDrawOrnamental(x1,y1,x2,y2,ornament: BYTE);
VAR
	i: INTEGER;
BEGIN
	ImagePut(LRLScreen^,LRLDecoration[ornament].Image[6].Data^,x1*10,y1*10,0,0,319,199);
	ImagePut(LRLScreen^,LRLDecoration[ornament].Image[7].Data^,x2*10,y1*10,0,0,319,199);
	ImagePut(LRLScreen^,LRLDecoration[ornament].Image[5].Data^,x1*10,y2*10,0,0,319,199);
	ImagePut(LRLScreen^,LRLDecoration[ornament].Image[8].Data^,x2*10,y2*10,0,0,319,199);
	FOR i:=x1+1 TO x2-1 DO BEGIN
		ImagePut(LRLScreen^,LRLDecoration[ornament].Image[3].Data^,i*10,y1*10,0,0,319,199);
		ImagePut(LRLScreen^,LRLDecoration[ornament].Image[4].Data^,i*10,y2*10,0,0,319,199);
	END;
	FOR i:=y1+1 TO y2-1 DO BEGIN
		ImagePut(LRLScreen^,LRLDecoration[ornament].Image[2].Data^,x1*10,i*10,0,0,319,199);
		ImagePut(LRLScreen^,LRLDecoration[ornament].Image[1].Data^,x2*10,i*10,0,0,319,199);
	END;
END;

PROCEDURE LRLRedrawLevel;
VAR
	i,j: INTEGER;
	s: STRING;
BEGIN
	ImageClear(LRLScreen^);
	FOR i:=1 TO 16 DO
		FOR j:=1 TO 30 DO
			WITH LRLLevel.Field[j,i] DO BEGIN
				IF (Flags AND 128)=0 THEN
					IF (Flags AND 32)<>0 THEN
						IF (Flags AND 16)=0 THEN
							ImagePut(LRLScreen^,LRLEnvironment[Image].Image[Count].Data^,j*10,i*10,0,0,319,199);
			END;
	FOR i:=1 TO 10 DO
		WITH LRLLevel.Player[i] DO BEGIN
			IF Controller<>0 THEN
				ImagePutTransparent(LRLScreen^,LRLFigure[Colour,SpriteData].Image[Sprite].Data^,
				Position.x*10+Position.xoffs,Position.y*10+Position.yoffs,0,0,319,199);
		END;
	FOR i:=1 TO 16 DO
		FOR j:=1 TO 30 DO
			WITH LRLLevel.Field[j,i] DO BEGIN
				IF (Flags AND 128)=0 THEN
					IF (Flags AND 32)=0 THEN
						IF (Flags AND 16)=0 THEN
							ImagePutTransparent(LRLScreen^,LRLEnvironment[Image].Image[LRLLevel.Field[j,i].Count].Data^,j*10,i*10,0,0,319,199);
			END;
	IF NOT Paused THEN BEGIN
		IF ShowScore THEN BEGIN
			STR(LRLScore,s);
			ImageStringGet(s,LRLFont^,LRLFontBuffer^,222);
			ImagePut(LRLScreen^,LRLFontBuffer^,56,185,0,0,319,199);
			ImageStringGet('Score: ',LRLFont^,LRLFontBuffer^,254);
			ImagePut(LRLScreen^,LRLFontBuffer^,10,185,0,0,319,199);
		END;
		IF ShowLives THEN BEGIN
			STR(LRLLives,s);
			ImageStringGet(s,LRLFont^,LRLFontBuffer^,222);
			ImagePut(LRLScreen^,LRLFontBuffer^,177,185,0,0,319,199);
			ImageStringGet('Lives: ',LRLFont^,LRLFontBuffer^,254);
			ImagePut(LRLScreen^,LRLFontBuffer^,135,185,0,0,319,199);
		END;
		IF ShowLevel THEN BEGIN
			STR(LRLCLevel,s);
			ImageStringGet(s,LRLFont^,LRLFontBuffer^,222);
			ImagePut(LRLScreen^,LRLFontBuffer^,292,185,0,0,319,199);
			ImageStringGet('Level: ',LRLFont^,LRLFontBuffer^,254);
			ImagePut(LRLScreen^,LRLFontBuffer^,250,185,0,0,319,199);
		END;
	END
	ELSE
	BEGIN
		ImageStringGet('Game now paused',LRLFont^,LRLFontBuffer^,254);
		ImagePut(LRLScreen^,LRLFontBuffer^,160-ImageSizex(LRLFontBuffer^) DIV 2,185,0,0,319,199);
	END;
	LRLDrawOrnamental(0,0,31,17,1);
END;

PROCEDURE LRLStartSequence;
VAR
	tmpScreen1: POINTER;
	tmpScreen2: POINTER;
	i: INTEGER;
BEGIN
	GETMEM(tmpScreen1,64000);
	GETMEM(tmpScreen2,49000);
	ImageFill(tmpScreen2^,300,160,0);
	LRLRedrawLevel;
	i:=0;
	WHILE i<100 DO BEGIN
		DataMove(LRLScreen^,tmpScreen1^,64000,0,0);
		ImagePut(tmpScreen1^,tmpScreen2^,10,10, 0, i, 319, 199-i);
		ScreenApply(tmpScreen1^);
		i:=i+4;
	END;
	ScreenApply(LRLScreen^);
	FREEMEM(tmpScreen1,64000);
	FREEMEM(tmpScreen2,49000);
END;

PROCEDURE LRLEndSequence;
VAR
	tmpScreen1: POINTER;
	tmpScreen2: POINTER;
	i: INTEGER;
BEGIN
	GETMEM(tmpScreen1,64000);
	GETMEM(tmpScreen2,49000);
	ImageFill(tmpScreen2^,300,160,0);
	LRLRedrawLevel;
	i:=100;
	WHILE i>0 DO BEGIN
		DataMove(LRLScreen^,tmpScreen1^,64000,0,0);
		ImagePut(tmpScreen1^,tmpScreen2^,10,10, 0, i, 319, 199-i);
		ScreenApply(tmpScreen1^);
		i:=i-4;
	END;
	ImagePut(LRLScreen^,tmpScreen2^,10,10, 0, 0, 319, 199);
	ScreenApply(LRLScreen^);
	FREEMEM(tmpScreen1,64000);
	FREEMEM(tmpScreen2,49000);
END;

{
game result:
1 - zamurovali
2 - poimali
10 - vse zdelano
50 - no more levels
60 - no human players
100 - esc was pressed
}
PROCEDURE LRLUpdatePlayers;
VAR
	i,j,k: INTEGER;
	spd: WORD;
BEGIN
	FOR i:=1 TO 10 DO BEGIN
		WITH LRLLevel.Player[i] DO BEGIN
			IF Controller<>0 THEN BEGIN
				IF (LRLLevel.Field[Position.x,Position.y].Flags AND 2<>0) THEN BEGIN
					IF i=1 THEN BEGIN
						EndOfGame:=TRUE;
						GameResult:=1;
						EXIT;
					END;
					IF Prizes<>0 THEN BEGIN
						Prizes:=0;
						LRLLevel.Field[Position.x,Position.y-1].Image:=4;
						LRLLevel.Field[Position.x,Position.y-1].Flags:=BrickFlags[8];
					END;
					REPEAT
						Position.y:=RANDOM(2)+1;
						Position.x:=RANDOM(30)+1;
					UNTIL (LRLLevel.Field[Position.x,Position.y].Image=1) OR
					(LRLLevel.Field[Position.x,Position.y].Image=2) OR
					(LRLLevel.Field[Position.x,Position.y].Image=3) OR
					(LRLLevel.Field[Position.x,Position.y].Image=4);
					Command:=10;
					CONTINUE;
				END;
				IF (LRLLevel.Field[Position.x,Position.y].Image=4) THEN BEGIN
					IF (Controller=2) THEN BEGIN
						IF Prizes=0 THEN BEGIN
							INC(Prizes);
							LRLLevel.Field[Position.x,Position.y].Image:=1;
							LRLLevel.Field[Position.x,Position.y].Flags:=BrickFlags[2];
						END;
					END
					ELSE
					BEGIN
						DEC(TotalPrizes);
						LRLScore:=LRLScore+100*LONGINT(LRLCLevel);
						LRLLevel.Field[Position.x,Position.y].Image:=1;
						LRLLevel.Field[Position.x,Position.y].Flags:=BrickFlags[2];
					END;
				END;
				IF (i=1) THEN BEGIN
					IF (TotalPrizes=0) AND (Position.y=1) AND
					(LRLLevel.Field[Position.x,Position.y].Image=2) THEN BEGIN
						EndOfGame:=TRUE;
						GameResult:=10;
						EXIT;
					END;
					FOR k:=2 TO 10 DO
						IF (LRLLevel.Player[k].Controller<>0) THEN
							IF (LRLLevel.Player[k].Position.x=Position.x) AND
								(LRLLevel.Player[k].Position.y=Position.y) THEN BEGIN
								EndOfGame:=TRUE;
								GameResult:=2;
								EXIT;
							END;
				END;
				IF (LRLLevel.Field[Position.x,Position.y].Flags AND 1<>0) THEN BEGIN
					IF (Controller=2) THEN BEGIN
						IF Prizes<>0 THEN BEGIN
							Prizes:=0;
							LRLLevel.Field[Position.x,Position.y-1].Image:=4;
							LRLLevel.Field[Position.x,Position.y-1].Flags:=BrickFlags[8];
						END;
					END;
				END;
				IF Controller=2 THEN
					spd:=2
				ELSE
					spd:=3;
				IF (LRLLevel.Field[Position.x,Position.y+1].Flags AND 4<>0) AND
				(LRLLevel.Field[Position.x,Position.y].Image<>3) AND
				((LRLLevel.Field[Position.x,Position.y].Image<>2) OR
				(LRLLevel.Field[Position.x,Position.y].Flags AND 16<>0)) AND
				(Position.y<16) THEN BEGIN
					FOR k:=2 TO 10 DO
						IF (k<>i) AND (LRLLevel.Player[k].Controller<>0) THEN
							IF (LRLLevel.Player[k].Position.x=Position.x) AND
								(LRLLevel.Player[k].Position.y=Position.y+1) AND
								(Position.y<16) THEN BEGIN
								k:=100;
								BREAK;
							END;
					IF k<>100 THEN BEGIN
						NewCommand:=5;
						NewCommandWas:=TRUE;
					END;
				END;
				IF NewCommandWas THEN BEGIN
					IF (NewCommand<>Command) AND (Command<>5) THEN BEGIN
						Command:=NewCommand;
						Sprite:=1;
					END;
					NewCommandWas:=FALSE;
				END;
				IF (Command=1) THEN BEGIN
					IF (LRLLevel.Field[Position.x,Position.y].Image=3) THEN BEGIN
						IF Position.xoffs<1 THEN BEGIN
							IF ((LRLLevel.Field[Position.x-1,Position.y].Flags AND 8=0) AND
							(LRLLevel.Field[Position.x-1,Position.y].Image<>3)) OR
							(LRLLevel.Field[Position.x,Position.y].Image<>3) OR (Position.x=1) THEN BEGIN
								Command:=10;
								Position.xoffs:=0;
							END;
						END;
						IF (Command<>10) AND (SpriteData<>6) THEN BEGIN
							SpriteData:=6;
							Sprite:=1;
						END;
					END
					ELSE
					BEGIN
						IF Position.xoffs<1 THEN BEGIN
							IF (LRLLevel.Field[Position.x-1,Position.y].Flags AND 8=0) OR (Position.x=1) THEN BEGIN
								Command:=10;
								Position.xoffs:=0;
							END;
						END;
						IF (Command<>10) AND (SpriteData<>1) THEN SpriteData:=1;
					END;
					IF Command<>10 THEN BEGIN
						k:=1;
						WHILE (k>0) DO BEGIN
							INC(k);
							IF k=11 THEN BEGIN
								IF (SpriteData=6) THEN BEGIN
									IF (Sprite=2) THEN
										DEC(Position.xoffs,5);
									IF (Sprite=3) THEN
										DEC(Position.xoffs,1);
								END
								ELSE
									DEC(Position.xoffs,spd);
								BREAK;
							END;
							IF (k<>i) AND (i<>1) AND (LRLLevel.Player[k].Controller<>0) THEN
								IF (LRLLevel.Player[k].Position.x=Position.x-1) AND
									(LRLLevel.Player[k].Position.y=Position.y) THEN BEGIN
									Command:=10;
									BREAK;
								END;
						END;
					END;
				END;
				IF (Command=2) THEN BEGIN
					IF (LRLLevel.Field[Position.x,Position.y].Image=3) THEN BEGIN
						IF Position.xoffs>-1 THEN BEGIN
							IF ((LRLLevel.Field[Position.x+1,Position.y].Flags AND 8=0) AND
							(LRLLevel.Field[Position.x+1,Position.y].Image<>3)) OR
							(LRLLevel.Field[Position.x,Position.y].Image<>3) OR (Position.x=30) THEN BEGIN
								Command:=10;
								Position.xoffs:=0;
							END;
						END;
						IF (Command<>10) AND (SpriteData<>7) THEN BEGIN
							SpriteData:=7;
							Sprite:=1;
						END;
					END
					ELSE
					BEGIN
						IF Position.xoffs>-1 THEN BEGIN
							IF (LRLLevel.Field[Position.x+1,Position.y].Flags AND 8=0) OR (Position.x=30) THEN BEGIN
								Command:=10;
								Position.xoffs:=0;
							END;
						END;
						IF (Command<>10) AND (SpriteData<>2) THEN SpriteData:=2;
					END;
					IF Command<>10 THEN BEGIN
						k:=1;
						WHILE (k>0) DO BEGIN
							INC(k);
							IF k=11 THEN BEGIN
								IF (SpriteData=7) THEN BEGIN
									IF (Sprite=2) THEN
										INC(Position.xoffs,5);
									IF (Sprite=3) THEN
										INC(Position.xoffs,1);
								END
								ELSE
									INC(Position.xoffs,spd);
								BREAK;
							END;
							IF (k<>i) AND (i<>1) AND (LRLLevel.Player[k].Controller<>0) THEN
								IF (LRLLevel.Player[k].Position.x=Position.x+1) AND
									(LRLLevel.Player[k].Position.y=Position.y) THEN BEGIN
									Command:=10;
									BREAK;
								END;
						END;
					END;
				END;
				IF (Command=3) THEN BEGIN
					IF Position.yoffs<1 THEN BEGIN
						IF ((LRLLevel.Field[Position.x,Position.y].Image<>2) OR
						(LRLLevel.Field[Position.x,Position.y].Flags AND 16<>0)) OR
						((LRLLevel.Field[Position.x,Position.y-1].Flags AND 4=0) AND
						((LRLLevel.Field[Position.x,Position.y-1].Image<>2) OR
						(LRLLevel.Field[Position.x,Position.y-1].Flags AND 16<>0))) OR
						(Position.y<2) THEN BEGIN
							Command:=10;
							Position.yoffs:=0;
						END;
					END;
					IF (Command<>10) AND (SpriteData<>3) THEN SpriteData:=3;
					IF Command<>10 THEN BEGIN
						k:=1;
						WHILE (k>0) DO BEGIN
							INC(k);
							IF k=11 THEN BEGIN
								DEC(Position.yoffs,spd);
								BREAK;
							END;
							IF (k<>i) AND (i<>1) AND (LRLLevel.Player[k].Controller<>0) THEN
								IF (LRLLevel.Player[k].Position.y=Position.y-1) AND
									(LRLLevel.Player[k].Position.x=Position.x) THEN BEGIN
									Command:=10;
									BREAK;
								END;
						END;
					END;
				END;
				IF (Command=4) THEN BEGIN
					IF (LRLLevel.Field[Position.x,Position.y].Image=3) AND
					((LRLLevel.Field[Position.x,Position.y+1].Image<>2) OR
					(LRLLevel.Field[Position.x,Position.y+1].Flags AND 16<>0)) AND
					(Position.y<16) THEN BEGIN
						Command:=5;
						Sprite:=1;
						IF (LRLLevel.Field[Position.x,Position.y+1].Flags AND 4<>0) THEN
							INC(Position.yoffs);
					END
					ELSE
					BEGIN
						IF Position.yoffs>-1 THEN BEGIN
							IF (((LRLLevel.Field[Position.x,Position.y+1].Image<>2) OR
							(LRLLevel.Field[Position.x,Position.y+1].Flags AND 16<>0)) AND
							(LRLLevel.Field[Position.x,Position.y+1].Flags AND 4=0)) OR
							(Position.y=16) THEN BEGIN
								Command:=10;
								Position.yoffs:=0;
							END;
						END;
						IF (Command<>10) AND (SpriteData<>4) THEN SpriteData:=4;
						IF Command<>10 THEN BEGIN
							k:=1;
							WHILE (k>0) DO BEGIN
								INC(k);
								IF k=11 THEN BEGIN
									INC(Position.yoffs,spd);
									BREAK;
								END;
								IF (k<>i) AND (i<>1) AND (LRLLevel.Player[k].Controller<>0) THEN
									IF (LRLLevel.Player[k].Position.y=Position.y+1) AND
										(LRLLevel.Player[k].Position.x=Position.x) THEN BEGIN
										Command:=10;
										BREAK;
									END;
							END;
						END;
					END;
				END;
				IF (Command=5) THEN BEGIN
					IF Position.yoffs<1 THEN BEGIN
						IF (LRLLevel.Field[Position.x,Position.y+1].Flags AND 4=0) OR (Position.y=16) OR
						(LRLLevel.Field[Position.x,Position.y].Image=3) OR
						((LRLLevel.Field[Position.x,Position.y].Flags AND 1<>0) AND (i<>1)) THEN BEGIN
							Command:=10;
							IF (LRLLevel.Field[Position.x,Position.y].Image=3) THEN SpriteData:=5;
							Position.yoffs:=0;
							Position.xoffs:=0;
						END;
						FOR k:=2 TO 10 DO
							IF (k<>i) AND (LRLLevel.Player[k].Controller<>0) THEN
								IF (LRLLevel.Player[k].Position.x=Position.x) AND
									(LRLLevel.Player[k].Position.y=Position.y+1) AND
									(LRLLevel.Field[Position.x,Position.y+1].Flags AND 1<>0) AND
									(Position.y<16) THEN BEGIN
									Command:=10;
									Position.yoffs:=0;
									BREAK;
								END;
					END;
					IF (Command<>10) AND (SpriteData<>5) THEN BEGIN
						SpriteData:=5;
						Sprite:=1;
					END;
					IF Command<>10 THEN BEGIN
						INC(Position.yoffs,2);
					END;
				END;
				IF (Command=6) THEN BEGIN
					IF (Position.y<16) AND (Position.x>1) AND (LRLLevel.Field[Position.x-1,Position.y+1].Image=9) AND
						(LRLLevel.Field[Position.x-1,Position.y+1].Flags AND 1=0) AND
						(((LRLLevel.Field[Position.x-1,Position.y].Image=1) OR
						(LRLLevel.Field[Position.x-1,Position.y].Flags AND 1<>0)) OR
						(LRLLevel.Field[Position.x-1,Position.y].Flags AND 16<>0)) THEN BEGIN
						NewCommandWas:=TRUE;
						FOR k:=2 TO 10 DO
							IF (k<>i) AND (LRLLevel.Player[k].Controller<>0) THEN
								IF (LRLLevel.Player[k].Position.x=Position.x-1) AND
									(LRLLevel.Player[k].Position.y=Position.y) THEN BEGIN
									NewCommandWas:=FALSE;
									BREAK;
								END;
						IF NewCommandWas THEN BEGIN
							LRLLevel.Field[Position.x-1,Position.y+1].Flags:=LRLLevel.Field[Position.x-1,Position.y+1].Flags OR 1;
							Position.xoffs:=0;
							SpriteData:=8;
							NewCommandWas:=FALSE;
						END;
					END;
					Command:=10;
				END;
				IF (Command=7) THEN BEGIN
					IF (Position.y<16) AND (Position.x<30) AND (LRLLevel.Field[Position.x+1,Position.y+1].Image=9) AND
						(LRLLevel.Field[Position.x+1,Position.y+1].Flags AND 1=0) AND
						(((LRLLevel.Field[Position.x+1,Position.y].Image=1) OR
						(LRLLevel.Field[Position.x+1,Position.y].Flags AND 1<>0)) OR
						(LRLLevel.Field[Position.x+1,Position.y].Flags AND 16<>0)) THEN BEGIN
						NewCommandWas:=TRUE;
						FOR k:=2 TO 10 DO
							IF (k<>i) AND (LRLLevel.Player[k].Controller<>0) THEN
								IF (LRLLevel.Player[k].Position.x=Position.x+1) AND
									(LRLLevel.Player[k].Position.y=Position.y) THEN BEGIN
									NewCommandWas:=FALSE;
									BREAK;
								END;
						IF NewCommandWas THEN BEGIN
							LRLLevel.Field[Position.x+1,Position.y+1].Flags:=LRLLevel.Field[Position.x+1,Position.y+1].Flags OR 1;
							Position.xoffs:=0;
							SpriteData:=9;
							NewCommandWas:=FALSE;
						END;
					END;
					Command:=10;
				END;
				IF (Command=1) OR (Command=2) THEN BEGIN
					IF Position.yoffs<0 THEN
						INC(Position.yoffs)
					ELSE
					IF Position.yoffs>0 THEN
						DEC(Position.yoffs);
				END;
				IF (Command=3) OR (Command=4) OR (Command=5) THEN BEGIN
					IF Position.xoffs<0 THEN
						INC(Position.xoffs)
					ELSE
					IF Position.xoffs>0 THEN
						DEC(Position.xoffs);
				END;
				IF Command<6 THEN BEGIN
					INC(Sprite);
					IF Sprite>LRLFigure[Colour,SpriteData].ImageCount THEN Sprite:=1;
					IF Position.xoffs<-4 THEN BEGIN
						DEC(Position.x);
						Position.xoffs:=10+Position.xoffs;
					END;
					IF Position.xoffs>5 THEN BEGIN
						INC(Position.x);
						Position.xoffs:=Position.xoffs-10;
					END;
					IF Position.yoffs<-4 THEN BEGIN
						DEC(Position.y);
						Position.yoffs:=10+Position.yoffs;
					END;
					IF Position.yoffs>5 THEN BEGIN
						INC(Position.y);
						Position.yoffs:=Position.yoffs-10;
					END;
				END;
			END;
		END;
	END;
END;

PROCEDURE LRLUpdateBricks;
VAR
	i,j,k: INTEGER;
BEGIN
	FOR i:=1 TO 16 DO
		FOR j:=1 TO 30 DO BEGIN
			IF LRLLevel.Field[j,i].Flags AND 1<>0 THEN BEGIN
				IF LRLLevel.Field[j,i].Count=1 THEN BEGIN
					LRLLevel.Field[j,i].Flags:=LRLLevel.Field[j,i].Flags AND $FF-2;
					LRLLevel.Field[j,i].Flags:=LRLLevel.Field[j,i].Flags OR 4+8;
				END;
				IF LRLLevel.Field[j,i].IdleCount=0 THEN BEGIN
					INC(LRLLevel.Field[j,i].Count);
					IF LRLLevel.Field[j,i].Count<6 THEN BEGIN
						FOR k:=2 TO 10 DO
							IF (LRLLevel.Player[k].Controller<>0) THEN
								IF (LRLLevel.Player[k].Position.x=j) AND
									(LRLLevel.Player[k].Position.y=i-1) THEN BEGIN
									LRLLevel.Field[j,i].Count:=13-LRLLevel.Field[j,i].Count;
									LRLLevel.Field[j,i].Flags:=LRLLevel.Field[j,i].Flags OR 2;
									LRLLevel.Field[j,i].Flags:=LRLLevel.Field[j,i].Flags AND $FE-4-8;
									LRLLevel.Field[j,i].Count:=1;
									BREAK;
								END;
					END;
					IF LRLLevel.Field[j,i].Count=6 THEN BEGIN
						LRLLevel.Field[j,i].IdleCount:=100;
					END;
				END
				ELSE
					DEC(LRLLevel.Field[j,i].IdleCount);
				IF LRLLevel.Field[j,i].Count=12 THEN BEGIN
					LRLLevel.Field[j,i].Flags:=LRLLevel.Field[j,i].Flags OR 2;
					LRLLevel.Field[j,i].Flags:=LRLLevel.Field[j,i].Flags AND $FE-4-8;
					LRLLevel.Field[j,i].Count:=1;
				END;
			END;
		END;
END;

PROCEDURE LRLComputerPlayer;
VAR
	k,l,m,f1,f2,i,j: INTEGER;
BEGIN
	IF ComputerTurn>=ComputerReaction THEN BEGIN
		ComputerTurn:=0;
		FOR k:=1 TO 10 DO BEGIN
			WITH LRLLevel.Player[k] DO BEGIN
				IF Controller=2 THEN BEGIN
					NewCommandWas:=TRUE;
					NewCommand:=10;
					IF (Position.y>LRLLevel.Player[1].Position.y) THEN BEGIN
						IF ((LRLLevel.Field[Position.x,Position.y].Image=2) AND
						(LRLLevel.Field[Position.x,Position.y].Flags AND 16=0) AND
						((LRLLevel.Field[Position.x,Position.y-1].Image=2) OR
						(LRLLevel.Field[Position.x,Position.y-1].Flags AND 4<>0)) AND
						(Position.y>1)) THEN BEGIN
							NewCommand:=3;
						END
						ELSE
						BEGIN
							m:=1;
							l:=Position.x;
							i:=1;
							WHILE i<>0 DO BEGIN
								l:=l+i;
								IF ((LRLLevel.Field[l,Position.y].Image=2) AND
								(LRLLevel.Field[l,Position.y].Flags AND 16=0)) AND
								((LRLLevel.Field[l,Position.y-1].Image=2) AND
								(LRLLevel.Field[l,Position.y-1].Flags AND 16=0)) AND
								(Position.y<>1) THEN BEGIN
									IF m=0 THEN BEGIN
										f2:=Position.x-l;
										BREAK;
									END;
									m:=0;
									i:=NOT i+1;
									f1:=l-Position.x;
									l:=Position.x;
								END
								ELSE
								IF (LRLLevel.Field[l,Position.y].Flags AND 8=0) OR (l>30) OR (l<1) THEN BEGIN
									IF m=0 THEN BEGIN
										f2:=100;
										BREAK;
									END;
									m:=0;
									i:=NOT i+1;
									l:=Position.x;
									f1:=100;
								END;
							END;
							IF (f1=100) AND (f2=100) THEN
								NewCommand:=10
							ELSE
							BEGIN
								IF f1>f2 THEN
									NewCommand:=1
								ELSE
									NewCommand:=2;
							END;
						END;
					END
					ELSE
					IF (Position.y<LRLLevel.Player[1].Position.y) THEN BEGIN
						IF (((LRLLevel.Field[Position.x,Position.y+1].Image=2) AND
						(LRLLevel.Field[Position.x,Position.y+1].Flags AND 16=0)) OR
						(LRLLevel.Field[Position.x,Position.y+1].Flags AND 4<>0)) AND
						(Position.y<16) AND (LRLLevel.Field[Position.x,Position.y+1].Flags AND 1=0) THEN BEGIN
							NewCommand:=4;
						END
						ELSE
						BEGIN
							m:=1;
							l:=Position.x;
							i:=1;
							WHILE i<>0 DO BEGIN
								l:=l+i;
								IF ((LRLLevel.Field[l,Position.y+1].Image=2) AND
								(LRLLevel.Field[l,Position.y+1].Flags AND 16=0)) OR
								((LRLLevel.Field[l,Position.y+1].Flags AND 4<>0) AND
								(LRLLevel.Field[l,Position.y+1].Flags AND 1=0)) THEN BEGIN
									IF m=0 THEN BEGIN
										f2:=Position.x-l;
										BREAK;
									END;
									m:=0;
									i:=NOT i+1;
									f1:=l-Position.x;
									l:=Position.x;
								END
								ELSE
								IF (LRLLevel.Field[l,Position.y].Flags AND 8=0) OR (l>30) OR (l<1) THEN BEGIN
									IF m=0 THEN BEGIN
										f2:=100;
										BREAK;
									END;
									m:=0;
									i:=NOT i+1;
									l:=Position.x;
									f1:=100;
								END;
							END;
							IF (f1=100) AND (f2=100) THEN
								NewCommand:=10
							ELSE
							BEGIN
								IF f1>f2 THEN
									NewCommand:=1
								ELSE
									NewCommand:=2;
							END;
						END;
					END
					ELSE
					BEGIN
						IF (Position.x>LRLLevel.Player[1].Position.x) THEN
							NewCommand:=1;
						IF (Position.x<LRLLevel.Player[1].Position.x) THEN
							NewCommand:=2;
					END;
				END;
			END;
		END;
	END
	ELSE
		INC(ComputerTurn);
END;

PROCEDURE RefreshRunner;
INTERRUPT;
ASSEMBLER;
ASM
	DEC RefreshRemain
	JNZ @DoTimer

	MOV AX,RefreshDelay
	MOV RefreshRemain,AX
	MOV TimeToRefresh,-1

@DoTimer:
	PUSHF
	CALL OldTimer
END;

FUNCTION LRLPlayLevel(Number: BYTE): WORD;
VAR
	Keypress: WORD;
	i: WORD;
BEGIN
	RANDOMIZE;
	ComputerReaction:=1;
	LRLLoadLevel(Number);
	IF LRLCLevel<>Number THEN BEGIN
		GameResult:=50;
		EXIT;
	END;
	IF LRLLevel.Player[1].Controller<>1 THEN BEGIN
		GameResult:=60;
		EXIT;
	END;
	RefreshDelay:=1;
	RefreshRemain:=1;
	OldTimer:=SetInterrupt($8,@RefreshRunner);
	TimeToRefresh:=TRUE;
	GameStarted:=FALSE;
	GameResult:=0;
	Paused:=FALSE;
	EndOfGame:=FALSE;
	LRLStartSequence;
	Keypress:=0;
	REPEAT
		IF TimeToRefresh THEN BEGIN
			IF (GameStarted) AND (NOT Paused) THEN BEGIN
				LRLComputerPlayer;
				LRLUpdatePlayers;
				LRLUpdateBricks;
			END;
			LRLRedrawLevel;
			ScreenApply(LRLScreen^);
			TimeToRefresh:=FALSE;
		END;
		IF Keypressed THEN BEGIN
			Keypress:=Readkey;
			GameStarted:=TRUE;
			Paused:=FALSE;
			FOR i:=0 TO ControlNumber-1 DO BEGIN
				IF KeyboardControls[i*3+1]=Keypress THEN BEGIN
					LRLLevel.Player[KeyboardControls[i*3+2]].NewCommand:=KeyboardControls[i*3+3];
					LRLLevel.Player[KeyboardControls[i*3+2]].NewCommandWas:=TRUE;
				END;
			END;
			IF (LO(Keypress)=80) OR (LO(Keypress)=112) THEN BEGIN
				Paused:=TRUE;
			END;
		END;
	UNTIL (LO(Keypress)=27) OR (EndOfGame);
	SetInterrupt($8,OldTimer);
	IF EndOfGame THEN
		LRLEndSequence
	ELSE
		GameResult:=100;
END;

END.